package com.haiwanwan.util.thriftant;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Execute;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * @author Daniel
 */
public class ThriftcTask extends Task {

    private String javaOptions = null;
    private String cppOptions = null;
    private String pythonOptions = null;
    private String phpOptions = null;
    private File srcPath = null;
    private File outPath = null;
    private boolean failonerror = true;
    private List<File> thriftFiles;

    public void setJavaOptions(String javaOptions) {
        this.javaOptions = javaOptions;
    }

    public void setCppOptions(String cppOptions) {
        this.cppOptions = cppOptions;
    }

    public void setPythonOptions(String pythonOptions) {
        this.pythonOptions = pythonOptions;
    }

    public void setPhpOptions(String phpOptions) {
        this.phpOptions = phpOptions;
    }

    public void setFailOnError(boolean failonerror) {
        this.failonerror = failonerror;
    }

    public void setSrcPath(File iPath) {
        this.srcPath = iPath;
    }

    public void setOutPath(File outPath) {
        this.outPath = outPath;
    }

    public void execute() {
        try {
            thriftFiles = new ArrayList<>();
            getThriftFiles(srcPath);
            for (File thriftFile : thriftFiles) {
                String[] commandLine = getCommandLine(thriftFile);
                Execute exe = new Execute();
                exe.setCommandline(commandLine);
                int errorcode = exe.execute();
                if (failonerror && (errorcode != 0)) {
                    throw new BuildException("Thrift code generation failed");
                }
            }
        } catch (IOException e) {
            log("Cannot generate code:" + e.getMessage(), Project.MSG_ERR);
            throw new BuildException("Some Exception runing the thrift compiler", e);
        }
    }

    private String[] getCommandLine(File thriftFile) throws IOException {
        String[] commandline;
        List<String> cmd = new LinkedList<>();
        cmd.add("thrift");
        cmd.add("-o");
        cmd.add(outPath.getAbsolutePath());

        log("Checking Depending Entries done",Project.MSG_VERBOSE);
        boolean canProceed = false;
        if (javaOptions != null) {
            cmd.add("--gen");
            cmd.add("java:" + javaOptions);
            canProceed = true;
        }
        if (cppOptions != null) {
            cmd.add("--gen=");
            cmd.add("cpp:" + cppOptions);
            canProceed = true;
        }
        if (pythonOptions != null) {
            cmd.add("--gen");
            cmd.add("py:" + pythonOptions);
            canProceed = true;
        }
        if (phpOptions != null) {
            cmd.add("--gen");
            cmd.add("php:" + phpOptions);
            canProceed = true;
        }
        if (thriftFile != null && thriftFile.exists()) {
            cmd.add(thriftFile.getAbsolutePath());
            canProceed = true;
        }
        StringBuilder sb = new StringBuilder();
        for (String c : cmd) {
            sb.append(c);
            sb.append(' ');
        }
        log(sb.toString(), Project.MSG_INFO);
        if (!canProceed) throw new BuildException("Cannot proceed without a '--gen' option");
        commandline = cmd.toArray(new String[cmd.size()]);
        return commandline;
    }

    private void getThriftFiles(File rootDir) {
        File[] files = rootDir.listFiles();
        if (files == null) return;
        for (File f : files) {
            if (f.isDirectory() ) {
                getThriftFiles(f);
            } else if (f.getName().endsWith(".thrift")) {
                thriftFiles.add(f);
            }
        }
    }
}
